// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: CorePosData
// Author: Richard Saxton
//
// Description: Methods for accessing the DSS and position information
//               from the spectrum
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: CorePosData.h,v 1.19 2013/02/15 15:04:00 rsaxton Exp $

#ifndef CorePosData_H
#define CorePosData_H

/* General */

/* SAS-specific classes */
#include "Xmm.h"
#include "CalCommon.h"
#include "CalServer.h"
#include "CcdQuantumDataServer.h"
#include "Tracer.h"
#include "DataSubspace.h"
#include "DatafileAttributes.h"
#include "SelectUtils.h"

/* Local classes */
#include "PixCoords.h"
#include "PositionUtils.h"

class CorePosData 
{
public:

  /** Constructors  */
  CorePosData( DataSet * spectrumset , Table * spectrumtable,  DatafileAttributes * specAttributes );

  /**  Destructors  */
  ~CorePosData();

  /* Get the centre of the source box in Raw pixels */
  PixCoords centre_raw()  const;

  /* Get the centre of the source box in detector pixels */
  CoordPair centre_detcoord()  const;

  /* Get the centre of the source box in theta, phi coords */
  TelCoords centre_telcoord()  const;

  /* Get the CCD number of the centre of the region */
  int centre_CCD() const;

  /* Get the pattern code */
  EventGrades patternCode()  const;

  /* Dummy a spatial region if timing or burst mode */
  void addDetRegion( PositionUtils * putils );

  /* Set and return the time ranges from the DSS */
  GtiList setTimeList(DataSet * spectrumset) const;
  GtiList getTimeList() const;

private:

  /** Members */

  DataSubspace  * _dss;

  Xmm::Instrument       _instrumentID;

  const PosWCS *      _poswcs;

  PositionUtils * _positionUtils;

  GtiList _timeList;
  
  /* Source position in detector coordinates */
  double _sourcedetx;
  double _sourcedety;

};
#endif
