// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: DumpVector
// Author: Hassan Siddiqui
//
// Description: Dumps an STL Vector into a DAL dataset 
// for later viewing by dsplot. Useful for debugging.
//
// Modifications: See ChangeLog
// 
// $Id: DumpVector.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef DUMPVECTOR_H
#define DUMPVECTOR_H

// General classes
#include <iostream>
#include <vector>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "Tracer.h"
#include "stringutil.h"

// Local classes

class DumpVector
{

public:

  static void go( const vector<double>& vect,
		  const string & outsetStr  = "vector_out.ds")
    {
      
      PROC;
  
      /* Open output dataset and column */

      DataSet * set   = dataSetServer->open(outsetStr, 
					    DataSet::Create);

      Table * table   = set->addTable("DMP_VECTOR", vect.size());

      Column * column = table->addColumn("VECTOR",
					 Column::Real32, 
					 "Dumped Vector");

      ColumnData * coldata = column->data();

      real32 * r32 = coldata->real32Data();

      /* Copy vector to column */
      unsigned int index = 0;
      for (vector<double>::const_iterator i = vect.begin();
	   i != vect.end();
	   ++i)
	{
	  //cout << "vect ["<< index << "] " << *i << endl;
	  r32[index] = *i;
	  ++index;
	}
    
      dataSetServer->close(set);

      ENDPROC;
    } 

};

#endif
