// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by
//            pkgmaker 0.1
//            Thursday, December 11 1997
//
// Module: GenericResponse
// Author: Hassan Siddiqui
//
// Description:
// Abstract interface to various flavours of RMF
// data being used as part of this package.
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: GenericResponse.h,v 1.18 2009/09/03 14:18:07 aibarra Exp $

#ifndef GenericResponse_H
#define GenericResponse_H

// General classes

#include <string>
#include <iostream>
#include <vector>

// SAS-specific classes
#include "CalServer.h" // IntervalVector defined here
#include "Dal.h"
#include "DetectorMap.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------
// class GenericResponse

// Abstract interface to various flavours of RMF
// data being used as part of this package.

// -----------------------------------------------------------------

class GenericResponse
{

public:

  /* Constructor */

  GenericResponse(){};

  virtual ~GenericResponse(){};

  /** Channel range accessors */
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => leading const removed
   */
  virtual  unsigned int   firstChannel()     const = 0;

  // CHANGE THIS: lastChannel() should be constructed from numberOfChannels.

  virtual  unsigned int   lastChannel()      const = 0;

  unsigned int           numberOfChannels() const
    {
      PROC;
      RETURN lastChannel() - firstChannel() + 1;
    }
/* AI gcc4 porting
 * end
 */
  /** Response Accessors */

  virtual unsigned int           ebinFirstIndex()   const {PROC; RETURN 0;}
  unsigned int                   ebinLastIndex()    const
    {
      PROC;
      RETURN ebinFirstIndex() + numberOfEbins() - 1;
    }

  virtual       unsigned int     numberOfEbins()    const = 0;
  //  virtual const IntervalVector<double> ebins()      const = 0;

  virtual const Interval<double> ebins( unsigned int row ) const = 0;

  // Return response vector according to row position
  virtual const vector<double>   responseVector    ( unsigned int energy_iterator ) const = 0;

  // Return response vector according to energy NB: result can be a bit fuzzy
  virtual const vector<double>   responseVector    ( double energy ) const = 0;

  // Return response vector according to ebin (convenience function)
  const vector<double>   responseVector  ( const Interval<double>& ebin ) const
    { PROC; RETURN  responseVector( 0.5 * (ebin.lo() + ebin.hi()) ); }



  /** Ebounds Accessors */
  /* Note the size and range corresponds to that specified for the channel,
   so no accessors are needed to cover this. */
  virtual const BinnedAxis * ebounds()  const = 0;

  /** Modifiers */
  /* Perform a spatial averaging of the response parameters */
  virtual void spatialAverage(DetectorMap *_detmap) const = 0;
};

#endif




