// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: NullResponseCorrector
// Author: Hassan Siddiqui
//
// Description: 
// Corrects response for pile-up
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: NullResponseCorrector.h,v 1.1 2013/02/15 15:05:54 rsaxton Exp $

#ifndef NullResponseCorrector_H
#define NullResponseCorrector_H

// General classes

#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "Tracer.h"

// Local Classes
#include "GenericResponse.h"
#include "ResponseCorrector.h"
#include "RmfUtils.h"

// -----------------------------------------------------------------------------------

// class NullResponseCorrector

// Applies NO correction to response.

// -----------------------------------------------------------------------------------

class NullResponseCorrector : public ResponseCorrector
{

public:
  
  /* Constructor */

  NullResponseCorrector(  const GenericResponse *  response ): 
    _response(response)
    {
    }


  /* Factory methods */

  const vector<double> correctedResponse( const Interval<double>& ebin0,
                                          const vector<double>& response ) const
    {
      PROC;

      vector<double> response0 = RmfUtils::normalizeResponse(response);

      RETURN response0;
    }
     
private:

  /* Data members */

  const GenericResponse * _response;

  /* Functions */

  // Suppress default constructor
  NullResponseCorrector();

  
};

#endif






