// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PattFracs
// Author: Richard Saxton
//
// Description: 
// Provides an clean interface to a fluxed spectrum or to an ARF
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: PattFracs.cc,v 1.2 2013/03/12 14:14:09 rsaxton Exp $

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"
#include "EnergySpectrum.h"
#include "PattFracs.h"
#include "CalOalUtility.h"
#include "CalServer.h"
#include "PixCoords.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class PattFracs

// Supports data consisting of an energy, energy width and data value

// -----------------------------------------------------------------------------------

 /* Constructor - from the RAW chip coords */
  //PattFracs::PattFracs(const unsigned short chipX, const unsigned short chipY, EnergySpectrum input_spectrum)
  PattFracs::PattFracs(PixCoords chipCoords)
    {
      PROC;

      // Get the pattern fraction object
       CcdQuantumDataServer *qds = 0; qds = calServer.getAtom(qds);
       Proxy<PatternFractionDataServer> pfds = qds->patternFractionData();

      // Set the required eventGrades
       vector<EventGrades> eventGrade;
       eventGrade.push_back(CombinedEventGrades::SINGLE); 
       eventGrade.push_back(CombinedEventGrades::DOUBLE); 
       eventGrade.push_back(CombinedEventGrades::TRIPLE); 
       eventGrade.push_back(CombinedEventGrades::QUADRUPLE); 

      // Set the size
       _N = eventGrade.size();
       for (unsigned i=0u; i < _N; i++) {

      // Get the pattern fraction at each energy, for this event grade
           _pcurve.push_back(pfds->at(chipCoords.x(), chipCoords.y()).
                                       grade(eventGrade[i]).inEnergySpace()); 
       }
    }

/* Destructor */
PattFracs:: ~PattFracs()
{
  PROC;
  ENDPROC;
};

//
// Method: fraction - return the pattern fraction for a given pattern and energy
//                         
double PattFracs::fraction(unsigned p, double energy) const
{

     return _pcurve[p](energy);
}


//
// Method: weightedFractions - returns the average fraction weighted by an
//                             input spectrum
//
vector<double> PattFracs::weightedFractions(EnergySpectrum spectrum)
const
{
     vector<double> wfrac;

    // Loop over each pattern type
     for (unsigned p=0; p < _N; p++) {

       // Loop over each energy in the input energy spectrum 
        double ptot=0.0;
        for (unsigned e=0; e<spectrum.energy().size(); ++e)
        {
         // Calculate the pattern fraction for this pattern and energy
         // and weight by the spectral value
          ptot += fraction(p,spectrum.energy()[e]) * spectrum.value(e);
        }
        wfrac.push_back(ptot/spectrum.sum());
     }

     return wfrac;
}
