// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PsfImage
// Author: Richard Saxton
//
// Description: Provides an interface to data consisting of a value 
//              indexed by an energy value
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: PsfImage.h,v 1.2 2013/03/05 11:25:49 rsaxton Exp $

#ifndef PsfImage_H
#define PsfImage_H

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"
#include "CalServer.h"
#include "EnergySpectrum.h"
#include "PattFracs.h"
#include "PixelIterator.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class PsfImage

// Supports data consisting of an energy, energy width and data value

// -----------------------------------------------------------------------------------

class PsfImage
{

public:

  /* Constructor from input vectors */
  PsfImage(EnergySpectrum input_spectrum, CoordPair halfWidthmm,
                   PattFracs *pfrac, double theta, double phi);

  PsfImage(double energy, PattFracs *pfrac,
                                     double theta, double phi);

  /* Destructor */
  ~PsfImage();


// Access methods
  Image<float> & image(int pattern);

  Image<float> normalise(Image<float> &image); // Normalise an input image to 1

  void normalise();  // Normalise the internal images to 1

 // Remove PSF pixels not within an input region
  void cleanRegion(PixelIterator * pxitr, CoordPair detoff);

private:

  void init(EnergySpectrum input_spectrum, CoordPair halfWidthmm,
                   PattFracs *pfrac, double theta, double phi);

  vector<Image<float> > _psfimage_pattern;
  unsigned _nx, _ny;

};


#endif




