// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: RmfDataServer
// Author: Hassan Siddiqui
//
// Description: Data pool which is used during arf generation
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: RmfDataServer.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef RmfDataServer_H
#define RmfDataServer_H

// General classes

#include <iostream>

// SAS-specific classes
#include "Dal.h"
#include "Xmm.h"


// General utility functions
#include "Tracer.h"

// Local classes
#include "GenericResponse.h"
#include "ResponseCorrector.h"
#include "PhotonDistribution.h"
#include "Frame.h"
#include "MatrixColumn.h"

// ===========================================================================

// class RmfDataServer

// Data pool which is used during rmf generation

// ===========================================================================

class RmfDataServer
{
public:

  /* Constructors */
  RmfDataServer(){};

  /* Destructors */
  virtual ~RmfDataServer()
    {
      errstr << "RmfDataServer destructor called. "
	     << Sas::message(Msg::App, Msg::Verbose);
    };

  /* Assessors */
  virtual       MatrixColumn      *  matrixColumn( Table * table,
						   const vector<unsigned long>& sizeVect ) const = 0;

  virtual       Xmm::Instrument      instrumentID()      const = 0;

  virtual const string               instrument()        const = 0;

  virtual const PhotonDistribution * photonDistibution() const = 0;

  virtual const GenericResponse    * response()          const = 0;

  virtual const ResponseCorrector  * responseCorrector() const = 0;

  virtual const Frame              * frame()             const = 0;

  /* Spectrum-related properties */
  virtual       unsigned             tlmin()             const = 0;

  virtual       unsigned             tlmax()             const = 0;

private:
  /* None. */
};

/* Reference to global RmfDataServer object, declared elsewhere. */
//extern RmfDataServer * rmfDataServer;
#endif
