// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: RmfTableIterator.h
// Author: Hassan Siddiqui
//
// Description: RMF generation task for the XMM Science Analysis System
// 
// Comments: 
//  This prototype is restricted the central CCD of the MOS1 detector.
//  
// Modifications: See ChangeLog
// 
// $Id: RmfTableIterator.h,v 1.18 2013/02/15 15:04:00 rsaxton Exp $

#ifndef RmfTableIterator_H
#define RmfTableIterator_H

// General classes

// SAS-specific classes
#include "Dal.h"

// Local classes
#include "RmfDataServer.h"
#include "MatrixColumn.h"

/*************************
 *
 * class RmfTableIterator
 *
 * Table interator class
 *
 *************************/

class RmfTableIterator //: public SubTableProcessor
{

public:

  /* Constructors */

  /* Suppress default constructor */
  RmfTableIterator();

  RmfTableIterator( RmfDataServer* rmfDataServer,
		    double threshold
		   );

  ~RmfTableIterator();

  /* Initialise table */
  void table(Table *);

  void setupTable(Table * table );

  void getGrouping();

  void processEntireTable(Table* table);

  /* Get row response for specified energy range */
  const vector<double> getResponse(const Interval<double>& ebin,
                                   const vector<double>& response);

private:

  RmfDataServer* _rmfDataServer;

  double        _threshold;

  ColumnData*   _ebinsloColData;

  ColumnData*   _ebinshiColData;

  ColumnData*   _n_grpColData;

  ColumnData*   _f_chanColData;

  ColumnData*   _n_chanColData;
  
  MatrixColumn * _matrixColumn;

  unsigned long _max_ngroups ;

  unsigned long _max_nresponse_elements;

  //vector<vector<double> > * _responseMatrix;
  vector<vector<double> > _responseMatrix;

};

#endif
