// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by
//            pkgmaker 0.1
//            Thursday, December 11 1997
//
// Module: RmfUtils
// Author: Hassan Siddiqui
//
// Description:
// General RmfUtility functions
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: RmfUtils.h,v 1.20 2013/04/17 14:26:09 rsaxton Exp $

#ifndef RmfUtils_H
#define RmfUtils_H

// General classes

#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "Tracer.h"

// Utility Classes
#include "NumericalRange.h"

// -----------------------------------------------------------------------------------

// class RmfUtils

// Suite of utility functions.
// Note that some/all of these should be upgraded to general STL Vector functions later.

// -----------------------------------------------------------------------------------

class RmfUtils
{

public:

  static double responseSum(const vector<double>& response)
    {
      PROC;

      assert ( response.size() > 0);

      double sum = 0;
      for (vector<double>::const_iterator itr_response = response.begin();
	   itr_response != response.end();
	   ++itr_response)
	{
	  sum += *itr_response;
	}

      RETURN sum;
    }

  static vector<double> multiplyResponse(const vector<double>& response1,
					 const vector<double>& response2)
    {
      PROC;

      assert ( response1.size() == response2.size() );

      vector<double> productResponse;

      // Clear out productResponse object
      productResponse.erase(productResponse.begin(), productResponse.end());

      vector<double>::const_iterator itr_response2 = response2.begin();

      for (vector<double>::const_iterator itr_response1 = response1.begin();
	   itr_response1 != response1.end();
	   ++itr_response1)
	{
	  productResponse.push_back( (*itr_response1) * (*itr_response2) );

	  ++itr_response2;
	}

      assert ( response1.size() == productResponse.size() );

      RETURN productResponse;
    }

  static const vector<double> multiplyResponse(const vector<double>& response,
					       const double factor)
    {
      PROC;

      vector<double> productResponse;

      // Clear out productResponse object
      productResponse.erase(productResponse.begin(), productResponse.end());

      for (vector<double>::const_iterator itr_response = response.begin();
	   itr_response != response.end();
	   ++itr_response)
	{
	  productResponse.push_back( (*itr_response) * factor );
	}

      assert ( response.size() == productResponse.size() );

      RETURN productResponse;
    }


  static const vector<double> addResponse(const vector<double>& response1,
					  const vector<double>& response2)
    {
      PROC;

      assert ( response1.size() == response2.size() );

      vector<double> sumResponse;

      // Clear out sumResponse object
      sumResponse.erase(sumResponse.begin(), sumResponse.end());

      vector<double>::const_iterator itr_response2 = response2.begin();

      for (vector<double>::const_iterator itr_response1 = response1.begin();
	   itr_response1 != response1.end();
	   ++itr_response1)
	{
	  sumResponse.push_back( (*itr_response1) + (*itr_response2) );

	  ++itr_response2;
	}

      assert ( response1.size() == sumResponse.size() );

      RETURN sumResponse;
    }

  static const vector<double> normalizeResponse(const vector<double>& response)
    {
      PROC;

      double normalization = responseSum(response);

      // Make sure that it is not close to zero
      /* AI gcc4 porting
       *  error: suggest braces around empty body in an ‘if’ statement
       *  => error in the if statement. ";" removed
       *    if (normalization > FLT64_MIN); =>
       *      if (normalization > FLT64_MIN)
       */
      if (normalization > FLT64_MIN)
    	  ALTRETURNV( multiplyResponse(response, (1/normalization)));

      errstr << "The response has a normalisation of zero. Returning response uncorrected. " << Sas::message(Msg::App, Msg::Verbose);
//      errstr << "The response has a normalisation of zero. Returning response uncorrected. "
//	     << Sas::warning("zeroNormalization");
      RETURN response;
    }



private:

  /* Functions */

  // Suppress default constructor
  RmfUtils();


};

#endif






