// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// Module: VarMatrixColumn
// Author: Hassan Siddiqui
//
// Description:
// Variable length vector column representation of MatrixColumn.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: VarMatrixColumn.h,v 1.17 2007/06/06 12:42:14 eojero Exp $

#ifndef VarMatrixColumn_H
#define VarMatrixColumn_H

/* General */

/* SAS-specific */
#include "Dal.h"

/* Local */
#include "MatrixColumn.h"

/*******************************
 *
 * class VarMatrixColumn
 *
 *******************************/

class VarMatrixColumn : public MatrixColumn
{

public:

  /* Constructor */
  VarMatrixColumn( Table * table );

  /* Destructor */
  ~VarMatrixColumn();

  /* Write to a given position in row */
  void writeTo( unsigned int element,
		double value);

  /* Set the number of elements in row */
  void row( unsigned long row,
	    unsigned int  elements);

  /* The number of elements in row */
  unsigned int rowElements() const;

  /* Inform class that processing for current row is complete */
  void closeRow();

private:

  Column     *  _matrixCol;
  unsigned long _row;
  CellData  *   _cellData;
  unsigned int  _rowelements;
};

#endif
